@@ -4,13 +4,18 @@
 package Tripletail;
 use strict;
 use warnings;
+BEGIN{ our $_CHKNONLAZY=$ENV{PERL_DL_NONLAZY} }
+BEGIN{ our $_CHKDYNALDR=$INC{'DynaLoader.pm'} }
 use UNIVERSAL qw(isa);
+use File::Spec;
+use Data::Dumper;
 
-our $VERSION = '0.11_01';
+our $VERSION = '0.27_02';
 
 our $TL = Tripletail->__new;
 our @specialization = ();
 our $LOG_SERIAL = 0;
+our $LASTERROR;
 
 require Unicode::Japanese;
 
@@ -19,6 +24,17 @@ if($ENV{TL_COVER_TEST_MODE}) {
 	Devel::Cover->import(qw(-silent on -summary off -db ./cover_db -coverage statement branch condition path subroutine time +ignore ^/));
 }
 
+*errorTrap = \&_errorTrap_is_deprecated;
+sub _errorTrap_is_deprecated {
+	die "\$TL->errorTrap(..) is deprecated, use \$TL->trapEror(..)"
+	#&trapError;
+}
+
+if( $ENV{MOD_PERL} )
+{
+	&PreloadModperl;
+}
+
 1;
 
 # -----------------------------------------------------------------------------
@@ -47,35 +63,16 @@ sub import {
 
 		my $trap = $TL->{INI}->get(TL => 'trap', 'die');
 		if($trap ne 'none' && $trap ne 'die' && $trap ne 'diewithprint') {
-			die __PACKAGE__."#startCgi, invalid trap option [$trap].\n";
+			die __PACKAGE__."#import, invalid trap option [$trap].\n";
 		}
 
 
 		$TL->{trap} = $trap;
 
-		if($trap eq 'die') {
-			$SIG{__DIE__} = sub {
-				# スタックトレースを付け加えて再度dieする。
-				# それ以外の事はしない。
-				my $msg = shift;
-
-				die isa($msg, 'Tripletail::Error') ? $msg : $TL->newError(error => $msg);
-			};
-		} elsif($trap eq 'diewithprint') {
-			$SIG{__DIE__} = sub {
-				# エラーをヘッダと共に表示して再度dieする。
-				my $msg = shift;
-
-				if (isa($msg, 'Tripletail::Error')) {
-					die $msg;
-				}
-				
-				my $err = $TL->newError(error => $msg);
-				
-				print "Content-Type: text/plain\n\n$err";
-				
-				die $err;
-			};
+		if($trap =~ /^(die|diewithprint)$/ )
+		{
+			my $trap = $1;
+			$SIG{__DIE__} = \&__die_handler_for_startup;
 		}
 		*{"$callpkg1\::CGI"} = _gensym(); # dummy to avoid strict.
 	} else {
@@ -85,6 +82,44 @@ sub import {
 	}
 }
 
+sub PreloadModperl
+{
+	require Apache2::RequestRec;
+	require Apache2::RequestIO;
+	require Apache2::Const;
+	Apache2::Const->import(-compile => qw(OK REDIRECT));
+	require APR::Table;
+}
+
+sub __die_handler_for_startup
+{
+	my $msg = shift;
+	my $trap = shift || $TL->{trap};
+
+	if( isa($msg, 'Tripletail::Error') )
+	{
+		die $msg;
+	}
+	my $prev = $LASTERROR;
+	if( $prev && !ref($msg) && $msg =~ s/^\Q$prev\E(?=Compilation failed in require at )// )
+	{
+		$prev->{message} .= $msg;
+		die $prev;
+	}
+
+	my $err = $TL->newError(error => $msg);
+	$LASTERROR = $err;
+
+	if( $trap eq 'diewithprint' && $err->{appear} ne 'usertrap' )
+	{
+		# die-with-print時かつevalの外であればは,
+		# エラーをヘッダと共に表示する.
+		print "Content-Type: text/plain\n\n$err";
+	}
+
+	die $err;
+}
+
 # -----------------------------------------------------------------------------
 # Pod::Coverage内からロードされているかの判定.
 #  (Test::Pod::Cover用)
@@ -166,6 +201,7 @@ sub escapeTag {
 		die "TL#escapeTag, ARG[1]: got undef.\n";
 	}
 
+	$str = "$str"; # stringify.
 	$str =~ s/\&/\&amp;/g;
 	$str =~ s/</\&lt;/g;
 	$str =~ s/>/\&gt;/g;
@@ -183,6 +219,7 @@ sub unescapeTag {
 		die "TL#unescapeTag, ARG[1]: got undef.\n";
 	}
 
+	$str = "$str"; # stringify.
 	$str =~ s/\&lt;/</g;
 	$str =~ s/\&gt;/>/g;
 	$str =~ s/\&quot;/\"/g;
@@ -207,6 +244,7 @@ sub escapeJs {
 		die "TL#escapeJs, ARG[1]: got undef.\n";
 	}
 
+	$str = "$str"; # stringify.
 	$str =~ s/(['"\\])/\\$1/g;
 	$str =~ s/\r/\\r/g;
 	$str =~ s/\n/\\n/g;
@@ -222,6 +260,7 @@ sub unescapeJs {
 		die "TL#unescapeJs, ARG[1]: got undef.\n";
 	}
 
+	$str = "$str"; # stringify.
 	$str =~ s/\\(['"\\])/$1/g;
 
 	$str;
@@ -235,6 +274,7 @@ sub encodeURL {
 		die "TL#encodeURL, ARG[1]: got undef.\n";
 	}
 
+	$str = "$str"; # stringify.
 	$str =~ s/([^a-zA-Z0-9\-\_\.\!\~\*\'\(\)])/
 	'%' . sprintf('%02x', unpack("C", $1))/eg;
 
@@ -249,6 +289,7 @@ sub decodeURL {
 		die "TL#decodeURL, ARG[1]: got undef.\n";
 	}
 
+	$str = "$str"; # stringify.
 	$str =~ s/\%([a-zA-Z0-9]{2})/pack("C", hex($1))/eg;
 
 	$str;
@@ -262,6 +303,7 @@ sub escapeSqlLike {
 		die "TL#escapeSqlLike, ARG[1]: got undef.\n";
 	}
 
+	$str = "$str"; # stringify.
 	$str =~ s/\\/\\\\/g;
 	$str =~ s/\%/\\\%/g;
 	$str =~ s/\_/\\\_/g;
@@ -277,6 +319,7 @@ sub unescapeSqlLike {
 		die "TL#unescapeSqlLike, ARG[1]: got undef.\n";
 	}
 
+	$str = "$str"; # stringify.
 	$str =~ s/\\\%/\%/g;
 	$str =~ s/\\\_/\_/g;
 	$str =~ s/\\\\/\\/g;
@@ -284,25 +327,30 @@ sub unescapeSqlLike {
 	$str;
 }
 
+sub __die_handler_for_localeval
+{
+	# スタックトレースを付け加えて再度dieする。
+	# それ以外の事はしない。
+	my $msg = shift;
+
+	die isa($msg, 'Tripletail::Error') ? $msg : $TL->newError(error => $msg);
+}
+
 sub startCgi {
 	my $this = shift;
 	my $param = { @_ };
 	
 	$this->{outputbuffering} = $this->INI->get(TL => 'outputbuffering', 0);
 
+	my $main_err;
 	eval {
 		# trap = diewithprint の場合はエラーハンドラを付け替える
 		# そうしないと Content-Type: text/plain が出力されてしまう。
 		if($this->{trap} eq 'diewithprint') {
-			$SIG{__DIE__} = sub {
-				# スタックトレースを付け加えて再度dieする。
-				# それ以外の事はしない。
-				my $msg = shift;
-
-				die isa($msg, 'Tripletail::Error') ? $msg : $TL->newError(error => $msg);
-			};
+			$SIG{__DIE__} = \&__die_handler_for_localeval;
 		}
 
+		# Tripletail::Debugをロード。debug機能が有効になっていれば、
 		# ここで各種フック類がインストールされる。
 		$this->getDebug;
 
@@ -333,6 +381,11 @@ sub startCgi {
 		if(!$this->getInputFilter) {
 			$this->setInputFilter('Tripletail::InputFilter::HTML');
 		}
+		if( $ENV{MOD_PERL} )
+		{
+			my $r = Apache2::RequestUtil->request;
+			$TL->{mod_perl} = { request => $r };
+		}
 
 		if($this->_getRunMode eq 'FCGI') {
 			# FCGIモードならメモリ監視フックとファイル監視フックをインストール
@@ -392,6 +445,7 @@ sub startCgi {
 				}
 
 				$this->__executeCgi($param->{-main});
+				$main_err = $@;
 
 				$request->Flush;
 
@@ -409,6 +463,7 @@ sub startCgi {
 			$this->log(TL => 'CGI mode');
 
 			$this->__executeCgi($param->{-main});
+			$main_err = $@;
 		}
 
 		$this->__executeHook('term');
@@ -433,26 +488,28 @@ sub startCgi {
 			$this->__dispError($err);
 		}
 	}
+	!$@ && $main_err and $@ = $main_err;
 
-	$this;
+	if( $ENV{MOD_PERL} )
+	{
+		Apache2::Const->OK;
+	}else
+	{
+		$this;
+	}
 }
 
-sub errorTrap {
+sub trapError {
 	my $this = shift;
 	my $param = { @_ };
 
+	my $main_err;
 	eval {
 		# trap = diewithprint の場合はエラーハンドラを付け替える
-		# そうしないと Content-Type: text/plain が出力されてしまう
-		local($SIG{__DIE__});
+		# そうしないと Content-Type: text/plain が出力されてしまう。
+		local($SIG{__DIE__}) = 'DEFAULT';
 		if ($this->{trap} eq 'diewithprint'){
-			$SIG{__DIE__} = sub {
-				# スタックトレースを付け加えて再度dieする。
-				# それ以外の事はしない。
-				my $msg = shift;
-
-				die isa($msg, 'Tripletail::Error') ? $msg : $TL->newError(error => $msg);
-			};
+			$SIG{__DIE__} = \&__die_handler_for_localeval;
 		}
 		# Tripletail::Debugをロード。debug機能が有効になっていれば、
 		# ここで各種フック類がインストールされる。
@@ -469,7 +526,7 @@ sub errorTrap {
 		}
 
 		if(!defined($param->{'-main'})) {
-			die __PACKAGE__."#errorTrap, -main handler was undef.\n";
+			die __PACKAGE__."#trapError, -main handler was undef.\n";
 		}
 
 		$this->__executeHook('init');
@@ -479,6 +536,7 @@ sub errorTrap {
 		eval {
 			$param->{'-main'}();
 		};
+		$main_err = $@;
 		if(my $err = $@) {
 			if($this->{trap} eq 'none') {
 				die $err;
@@ -503,9 +561,10 @@ sub errorTrap {
 		}
 
 		# このevalでキャッチされたという事は、-mainの外で例外が起きた。
-		$this->log(errorTrap => "Died outside the `-main': $err");
-		print STDERR "Died outside the `-main': $err\n";
+		$this->log(trapError => "Died outside the `-main': $err");
+		print STDERR __PACKAGE__."#trapError, Died outside the `-main': $err\n";
 	}
+	!$@ && $main_err and $@ = $main_err;
 
 	$this;
 }
@@ -550,22 +609,51 @@ sub dispatch {
 
 sub log {
 	my $this = shift;
-	my $group = shift;
-	my $log = shift;
+    my $group;
+    my $message;
+
+    my $stringify = sub {
+        my $val = shift;
+        
+        if (ref $val) {
+            Data::Dumper->new([$val])
+              ->Indent(1)->Purity(0)->Useqq(1)->Terse(1)->Deepcopy(1)
+                ->Quotekeys(0)->Sortkeys(1)->Deparse(1)->Dump;
+        }
+        else {
+            $val; # 元々スカラーだった
+        }
+    };
+    
+    if (@_ == 1) {
+        # "呼出し元ファイル名(行数):関数名"
+        my ($filename, $line) = (caller 0)[1, 2];
+        my $sub = (caller 1)[3];
+
+        $group   = sprintf '%s(%d) >> %s', $filename, $line, $sub;
+        $message = $stringify->(shift);
+    }
+    elsif (@_ == 2) {
+        $group   = shift;
+        $message = $stringify->(shift);
+    }
+    else {
+        die "TL#log, invalid call of \$TL->log().\n";
+    }
 
 	if(!defined($group)) {
 		die "TL#log, ARG[1]: got undef.\n";
 	}
-	if(!defined($log)) {
+	if(!defined($message)) {
 		die "TL#log, ARG[2]: got undef.\n";
 	}
 
 	$this->getDebug->_tlLog(
 		group => $group,
-		log   => $log,
+		log   => $message,
 	);
 
-	$this->_log($group, $log);
+	$this->_log($group, $message);
 }
 
 sub _log {
@@ -604,7 +692,7 @@ sub _log {
 		. sprintf('%04d%02d/%02d-%02d.log', @localtime[5,4,3,2]);
 
 	if(!exists($this->{cacheLogPath}) || !defined($dirstat[1]) || $path ne $this->{cacheLogPath}) {
-
+		# month is changed.
 		delete $this->{cacheLogFh};
 		my $umask = umask(0);
 		local($@);
@@ -632,6 +720,7 @@ sub _log {
 	my @stat = stat($path);
 	if(!defined($this->{cacheLogFh}) || !defined($stat[1]) || ($this->{cacheLogInode} != $stat[1])) {
 
+		# hour is changed.
 		my $fh = $this->_gensym;
 		if(!open($fh, ">>$path")) {
 			print "Content-Type: text/plain\n\n";
@@ -648,6 +737,15 @@ sub _log {
 		my @newstat = stat($path);
 		$this->{cacheLogFh} = $fh;
 		$this->{cacheLogInode} = $newstat[1];
+		local($@);
+		eval
+		{
+			my $rel_to_logfile = sprintf('%04d%02d/%02d-%02d.log', @localtime[5,4,3,2]);
+			local($SIG{__DIE__});
+			my $cur_linkfile = File::Spec->catfile($this->{logdir}, "current");
+			unlink($cur_linkfile);
+			symlink($rel_to_logfile, $cur_linkfile);
+		};
 	}
 
 	my $fh = $this->{cacheLogFh};
@@ -886,6 +984,12 @@ sub _sendErrorIfNeeded {
 	}
 }
 
+sub _hostname
+{
+	my $host = `hostname -f 2>&1` || `hostname 2>&1`;
+	$host && $host=~/^\s*([\w.-]+)\s*$/ ? $1 : '';
+}
+
 sub sendError {
 	my $this = shift;
 	my $opts = { @_ };
@@ -919,11 +1023,7 @@ sub sendError {
 	push @lines, '';
 	push @lines, '----';
 
-	my $host = '';
-	eval { $host = `hostname -f` };
-	if(!$host) {
-		eval { $host = `hostname` };
-	}
+	my $host = _hostname();
 	if($host) {
 		chomp $host;
 		unshift @lines, "HOST: $host";
@@ -1069,7 +1169,9 @@ sub parsePeriod {
 		} elsif(s/^days?//) {
 			commit(60 * 60 * 24);
 		} elsif(s/^mon(?:ths?)?//) {
-			commit(60 * 60 * 24 * 30);
+			commit(60 * 60 * 24 * 30.436875);
+		} elsif(s/^years?//) {
+			commit(60 * 60 * 24 * 365.2425);
 		} elsif(s/^(\d+)//) {
 			if(defined($lastnum)) {
 				die __PACKAGE__."#parsePeriod, invalid time string [$str]:".
@@ -1086,7 +1188,7 @@ sub parsePeriod {
 		commit(1);
 	}
 
-	$result;
+	int($result);
 }
 
 sub parseQuantity {
@@ -1299,8 +1401,14 @@ sub newTagCheck {
 
 sub newTemplate {
 	my $this = shift;
-
-	require Tripletail::Template;
+	
+	my $err;
+	{
+		local($@);
+		eval{ require Tripletail::Template; };
+		$err = $@;
+	}
+	$err and die $err;
 
 	Tripletail::Template->_new(@_);
 }
@@ -1334,12 +1442,16 @@ sub newError {
 
 	# Tripletail::Error のロード失敗は特別に扱わなければならない。
 	# die ハンドラがこれを利用する為である。
-	eval {
-		require Tripletail::Error;
-	};
-	if ($@) {
-		print STDERR $@;
-		exit 1;
+	if( !Tripletail::Error->can("_new") )
+	{
+		local($@);
+		eval {
+			require Tripletail::Error;
+		};
+		if ($@) {
+			print STDERR $@;
+			exit 1;
+		}
 	}
 
 	Tripletail::Error->_new(@_);
@@ -1464,20 +1576,59 @@ sub watch {
 }
 
 sub dump {
+    # dump($group, $obj)
+    # dump($group, $obj, $level)
+    # dump($obj)
+    # dump($obj, $level)
 	my $this = shift;
-	my $group = shift;
-	my $var = shift;
-	my $level = shift || 0;
+    my $group;
+    my $val;
+    my $level;
 
-	local($@);
-	eval {
-		use Data::Dumper;
-		my $dump = Data::Dumper->new([$var])
-			->Maxdepth($level)->Indent(1)->Terse(1)->Dump;
-		$this->log($group => $dump);
-	};
+    my $auto_group = sub {
+        # "呼出し元ファイル名(行数):関数名"
+        my ($filename, $line) = (caller 1)[1, 2];
+        my $sub = (caller 2)[3];
 
-	$this;
+        sprintf '%s(%d) >> %s', $filename, $line, $sub;
+    };
+
+    if (@_ == 0 || @_ > 3) {
+        die __PACKAGE__."#dump, invalid call of \$TL->dump().\n";
+    }
+    elsif (@_ == 1) {
+        $group = $auto_group->();
+        $val   = shift;
+        $level = 0;
+    }
+    elsif (@_ == 2) {
+        if (ref $_[0]) {
+            # dump($obj, $level)
+            $group = $auto_group->();
+            $val   = shift;
+            $level = shift;
+        }
+        else {
+            # dump($group, $obj)
+            $group = shift;
+            $val   = shift;
+            $level = 0;
+        }
+    }
+    elsif (@_ == 3) {
+        $group = shift;
+        $val   = shift;
+        $level = shift;
+    }
+    else {
+        die "Internal error";
+    }
+
+    my $dump = Data::Dumper->new([$val])
+      ->Indent(1)->Purity(0)->Useqq(1)->Terse(1)->Deepcopy(1)
+        ->Quotekeys(0)->Sortkeys(1)->Deparse(1)->Maxdepth($level)->Dump;
+
+    $this->log($group => $dump);
 }
 
 sub printCacheUnlessModified {
@@ -1886,9 +2037,9 @@ servers = localhost:11211
 
   #topはキャッシュするキー。画面毎にキーを設定する。ページャーなどを利用する場合、画面毎になる点を注意する（page-1等にする）
   #キャッシュにヒットした場合、時間を比較して、304でリダイレクトするか、メモリから読み込んで表示する
-  #returnした後にprintやflushなど出力する操作は不可なため注意する事
+  #printCacheUnlessModifiedでundefが返ってきた後は、printやflushなど出力する操作は不可なため注意する事
   return if(!defined($TL->printCacheUnlessModified('top')));
-  #キャッシュすることを宣言する。なお、宣言はprintCacheUnlessModifiedより後であれば、どの時点で行ってもかまわない
+  #キャッシュすることを宣言する。なお、宣言はprintCacheUnlessModifiedより後でprintより前であれば、どの時点で行ってもかまわない
   $TL->setCache('top');
 
   #実際のスクリプトを記述し、出力を行う
@@ -1904,7 +2055,7 @@ servers = localhost:11211
 
 =item 読み込み側例（ユーザー名等一部に固有の情報を埋め込んでいる場合）
 
-  #クッキーデータの取得、クッキーに固有の情報を入れておくと高速に動作出来る（DB等から読み込みTL:Formクラスにセットしても可）
+  #クッキーデータの取得、クッキーに固有の情報を入れておくと高速に動作出来る（DB等から読み込みTripletail::Formクラスにセットしても可）
   my $cookiedata = $TL->getCookie->get('TLTEST');
   $cookiedata->set('<&NAME>' => $name) if(!$cookiedata->exists('name'));
   $cookiedata->set('<&POINT>' => $point) if(!$cookiedata->exists('point'));
@@ -1970,7 +2121,7 @@ C<use Tripletail> したスクリプトファイルや、その L<Ini|Tripletail
 CGIでない一般のスクリプトとしてプログラムを動作させるモード。
 CGIモード特有の機能は利用出来ない。
 
-このモードでは L<< $TL->errorTrap|/"errorTrap" >> メソッドで L</"Main関数"> を呼ぶ。
+このモードでは L<< $TL->trapError|/"trapError" >> メソッドで L</"Main関数"> を呼ぶ。
 
 =back
 
@@ -2003,7 +2154,7 @@ L<< $TL->setHook|/"setHook" >> メソッドを用いてフックを掛ける事
 
 =item init
 
-L</"startCgi"> もしくは L</"errorTrap"> が呼ばれ、最初に L</"Main関数"> が
+L</"startCgi"> もしくは L</"trapError"> が呼ばれ、最初に L</"Main関数"> が
 呼ばれる前。
 
 =item preRequest
@@ -2017,81 +2168,16 @@ L</"Main関数"> が呼ばれた直後。
 =item term
 
 最後に L</"Main関数"> が呼ばれた後。termフック呼出し後に L</"startCgi">
-もしくは L</"errorTrap"> が終了する。
+もしくは L</"trapError"> が終了する。
 
 =back
 
 
 =head2 METHODS
 
-=over 4
+=head3 よく使うもの
 
-=item C<< INI >>
-
-  $TL->INI
-
-C<< use Tripletail qw(filename.ini); >> で読み込まれた L<Tripletail::Ini> を返す。
-
-=item C<< CGI >>
-
-  $TL->CGI
-  $CGI
-
-リクエストを受け取った L<Tripletail::Form> オブジェクトを返す。
-また、このオブジェクトは startCgi メソッドの呼び出し元パッケージに export される。
-
-このメソッドがundefでない値を返すのは、 L</"preRequest"> フックが呼ばれる
-直前から L</"postRequest"> フックが呼ばれた直後までである。
-
-=item C<< escapeTag >>
-
-  $result = $TL->escapeTag($value)
-
-&E<lt>E<gt>"' の文字をエスケープ処理した文字列を返す。
-
-=item C<< unescapeTag >>
-
-  $result = $TL->unescapeTag($value)
-
-&E<lt>E<gt>"'&#??;&#x??; にエスケープ処理された文字を元に戻した文字列を返す。
-
-=item C<< escapeJs >>
-
-  $result = $TL->escapeJs($value)
-
-'"\ の文字を \ を付けてエスケープし，'\r' '\n' について '\\r' '\\n' に置き換える。
-
-=item C<< unescapeJs >>
-
-  $result = $TL->unescapeJs($value)
-
-escapeJs した文字列を元に戻す。
-
-=item C<< encodeURL >>
-
-  $result = $TL->encodeURL($value)
-
-文字列をURLエンコードした結果を返す。
-
-=item C<< decodeURL >>
-
-  $result = decodeURL($value)
-
-URLエンコードを解除し元に戻した文字列を返す。
-
-=item C<< escapeSqlLike >>
-
-  $result = $TL->escapeSqlLike($value)
-
-% _ \ の文字を \ でエスケープ処理した文字列を返す。
-
-=item C<< unescapeSqlLike >>
-
-  $result = $TL->unescapeSqlLike($value)
-
-\% \_ \\ にエスケープ処理された文字を元に戻した文字列を返す。
-
-=item C<< startCgi >>
+=head4 C<< startCgi >>
 
   $TL->startCgi(
     -main        => \&Main,    # メイン関数
@@ -2117,19 +2203,20 @@ C<Session> は、次のように配列へのリファレンスを渡す事で、
     -Session => ['Session1', 'Session2'],
   );
 
-=item C<< errorTrap >>
+通常スクリプトでは L</trapError> を参照.
 
-  $TL->errorTrap(
-    -main => \&Main, # メイン関数
-    -DB   => 'DB',   # DBを使う場合，iniのグループ名を指定
-  );
+=head4 C<< CGI >>
 
-環境を整え、 L</"Main関数"> を実行する。
-L</"Main関数"> がdieした場合は、エラー内容が標準エラーへ出力される。
+  $TL->CGI
+  $CGI
 
-L</"startCgi"> と同様に、C<DB> には配列へのリファレンスを渡す事も出来る。
+リクエストを受け取った L<Tripletail::Form> オブジェクトを返す。
+また、このオブジェクトは startCgi メソッドの呼び出し元パッケージに export される。
 
-=item C<< dispatch >>
+このメソッドがundefでない値を返すのは、 L</"preRequest"> フックが呼ばれる
+直前から L</"postRequest"> フックが呼ばれた直後までである。
+
+=head4 C<< dispatch >>
 
   $result = $TL->dispatch($value, default => $default, onerror => \&Error)
 
@@ -2156,102 +2243,7 @@ onerrorが設定されていた場合、関数が存在しなければ onerror
       ...
   }
 
-=item C<< log >>
-
-  $TL->log($group => $log)
-
-ログを記録する。グループとログデータの２つを受け取る。
-
-ログにはヘッダが付けられ、ヘッダは「時刻(epoch値の16進数8桁表現) プロセスIDの16進数4桁表現 FastCGIのリクエスト回数の16進数4桁表現 [グループ]」の形で付けられる。
-
-=item C<< getLogHeader >>
-
-  my $logid = $TL->getLogHeader
-
-ログを記録するときのヘッダと同じ形式の文字列を生成する。
-「時刻(epoch値の16進数8桁表現) プロセスIDの16進数4桁表現 FastCGIのリクエスト回数の16進数4桁表現」の形の文字列が返される。
-
-=item C<< setHook >>
-
-  $TL->setHook($type, $priority, \&func)
-
-指定タイプの指定プライオリティのフックを設定する。
-既に同一タイプで同一プライオリティのフックが設定されていた場合、
-古いフックの設定は解除される。
-
-typeは、L</"init">, L</"term">, L</"preRequest">, L</"postRequest">
-の４種類が存在する。
-
-なお、1万の整数倍のプライオリティは Tripletail 内部で使用される。アプリ
-ケーション側で不用意に用いるとフックを上書きしてしまう可能性があるので
-注意する。
-
-=item C<< removeHook >>
-
-  $TL->removeHook($type, $priority)
-
-指定タイプの指定プライオリティのフックを削除する。
-
-=item C<< setContentFilter >>
-
-  $TL->setContentFilter($classname, %option)
-  $TL->setContentFilter([$classname, $priority], %option)
-  $TL->setContentFilter('Tripletail::Filter::HTML', charset => 'Shift_JIS')
-  $TL->setContentFilter(
-      'Tripletail::Filter::CSV', charset => 'Shift_JIS', filename => 'テストデータ.csv')
-
-L</"出力フィルタ"> を設定する。
-全ての出力の前に実行する必要がある。
-２番目の書式では、プライオリティを指定して独自のコンテンツフィルタを追加できる。
-省略時は優先度は1000となる。同一優先度のフィルタが既にセットされているときは、
-以前のフィルタ設定は解除される。
-
-返される値は、指定された L<Tripletail::Filter> のサブクラスのインスタンスである。
-
-設定したフィルタは、L</"preRequest"> のタイミングで保存され、
-L</"postRequest"> のタイミングで元に戻される。従って、L</"Main関数">内
-で setContentFilter を実行した場合、その変更は次回リクエスト時に持ち越
-されない。
-
-=item C<< getContentFilter >>
-
-  $TL->getContentFilter($priority)
-
-指定されたプライオリティのフィルタを取得する。省略時は1000となる。
-
-=item C<< removeContentFilter >>
-
-  $TL->removeContentFilter($priority)
-
-指定されたプライオリティのフィルタを削除する。省略時は1000となる。
-フィルタが１つもない場合は、致命的エラーとなり出力関数は使用できなくなる。
-
-=item C<< setInputFilter >>
-
-  $TL->setInputFilter($classname, %option)
-  $TL->setInputFilter([$classname, $priority], %option)
-
-L</"入力フィルタ"> を設定する。
-L</"startCgi"> の前に実行する必要がある。
-
-返される値は、指定された L<Tripletail::InputFilter> のサブクラスのインスタンスである。
-
-=item C<< getInputFilter >>
-
-  $TL->getInputFilter($priority)
-
-=item C<< removeInputFilter >>
-
-  $TL->removeInputFilter($priority)
-
-=item C<< sendError >>
-
-  $TL->sendError(-title => "タイトル", -error => "エラー")
-
-L<ini|Tripletail::Ini> で指定されたアドレスにエラーメールを送る。
-設定が無い場合は何もしない。
-
-=item C<< print >>
+=head4 C<< print >>
 
   $TL->print($str)
 
@@ -2261,120 +2253,64 @@ L<ini|Tripletail::Ini> で指定されたアドレスにエラーメールを送
 フィルタによってはバッファリングされる場合もあるが、
 基本的にはバッファリングされない。
 
-=item C<< location >>
+=head4 C<< location >>
 
   $TL->location('http://example.org/')
 
 CGIモードの時、指定されたURLへリダイレクトする。
 このメソッドはあらゆる出力の前に呼ばなくてはならない。 
 
-=item C<< parsePeriod >>
+=head3 変換処理
 
-  $TL->parsePeriod('10hour 30min')
+=head4 C<< escapeTag >>
 
-時間指定文字列を秒数に変換する。
-L</"度量衡"> を参照。
+  $result = $TL->escapeTag($value)
 
-=item C<< parseQuantity >>
+&E<lt>E<gt>"' の文字をエスケープ処理した文字列を返す。
 
-  $TL->parseQuantity('100mi 50ki')
+=head4 C<< unescapeTag >>
 
-量指定文字列を元の数に変換する。
-L</"度量衡"> を参照。
+  $result = $TL->unescapeTag($value)
 
-=item C<< getCookie >>
+&E<lt>E<gt>"'&#??;&#x??; にエスケープ処理された文字を元に戻した文字列を返す。
 
-L<Tripletail::Cookie> オブジェクトを取得。
+=head4 C<< escapeJs >>
 
-=item C<< getCsv >>
+  $result = $TL->escapeJs($value)
 
-L<Tripletail::CSV> オブジェクトを取得。
+'"\ の文字を \ を付けてエスケープし，'\r' '\n' について '\\r' '\\n' に置き換える。
 
-=item C<< newDateTime >>
+=head4 C<< unescapeJs >>
 
-L<Tripletail::DateTime> オブジェクトを作成。
+  $result = $TL->unescapeJs($value)
 
-=item C<< getDB >>
+escapeJs した文字列を元に戻す。
 
-  $DB = $TL->getDB($group)
+=head4 C<< encodeURL >>
 
-L<Tripletail::DB> オブジェクトを取得。
+  $result = $TL->encodeURL($value)
 
-=item C<< newDB >>
+文字列をURLエンコードした結果を返す。
 
-  $DB = $TL->newDB($group)
+=head4 C<< decodeURL >>
 
-L<Tripletail::DB> オブジェクトを作成。
+  $result = decodeURL($value)
 
-=item C<< getFileSentinel >>
+URLエンコードを解除し元に戻した文字列を返す。
 
-L<Tripletail::FileSentinel> オブジェクトを取得。
+=head4 C<< escapeSqlLike >>
 
-=item C<< newForm >>
+  $result = $TL->escapeSqlLike($value)
 
-L<Tripletail::Form> オブジェクトを作成。
+% _ \ の文字を \ でエスケープ処理した文字列を返す。
 
-=item C<< newHtmlFilter >>
+=head4 C<< unescapeSqlLike >>
 
-L<Tripletail::HtmlFilter> オブジェクトを作成。
+  $result = $TL->unescapeSqlLike($value)
 
-=item C<< newHtmlMail >>
+\% \_ \\ にエスケープ処理された文字を元に戻した文字列を返す。
 
-L<Tripletail::HtmlMail> オブジェクトを作成。
-
-=item C<< newIni >>
-
-L<Tripletail::Ini> オブジェクトを作成。
-
-=item C<< newMail >>
-
-L<Tripletail::Mail> オブジェクトを作成。
-
-=item C<< getMemorySentinel >>
-
-L<Tripletail::MemorySentinel> オブジェクトを取得。
-
-=item C<< newPager >>
-
-L<Tripletail::Pager> オブジェクトを作成。
-
-=item C<< getRawCookie >>
-
-L<Tripletail::RawCookie> オブジェクトを取得。
-
-=item C<< newSendmail >>
-
-L<Tripletail::Sendmail> オブジェクトを作成。
-
-=item C<< newSMIME >>
-
-L<Tripletail::SMIME> オブジェクトを作成。
-
-=item C<< newTagCheck >>
-
-L<Tripletail::TagCheck> オブジェクトを作成。
-
-=item C<< newTemplate >>
-
-L<Tripletail::Template> オブジェクトを作成。
-
-=item C<< getSession >>
-
-L<Tripletail::Session> オブジェクトを取得。
-
-=item C<< newValue >>
-
-L<Tripletail::Value> オブジェクトを作成。
-
-=item C<< newValidator >>
-
-L<Tripletail::Validator> オブジェクトを生成。
-
-=item C<< newMemCached >>
-
-L<Tripletail::MemCached> オブジェクトを生成。
-
-=item C<< charconv >>
+=head4 C<< charconv >>
 
   $str = $TL->charconv($str, $from, $to, $prefer_encode);
   $str = $TL->charconv($str, [$from1, $from2, ...], $to, $prefer_encode);
@@ -2400,14 +2336,240 @@ C<$to> が省略された場合は C<'UTF-8'> になる。
 確実に指定できる文字コードは、UTF-8，Shift_JIS，EUC-JP，ISO-2022-JP である。
 それ以外の場合は、L<Encode> と L<Unicode::Japanese> のどちらが使用されるかにより使用できるものが異なる。
 
-=item C<< readFile >>
+=head4 C<< parsePeriod >>
+
+  $TL->parsePeriod('10hour 30min')
+
+時間指定文字列を秒数に変換する。小数点が発生した場合は切り捨てる。
+L</"度量衡"> を参照。
+
+=head4 C<< parseQuantity >>
+
+  $TL->parseQuantity('100mi 50ki')
+
+量指定文字列を元の数に変換する。
+L</"度量衡"> を参照。
+
+=head3 インスタンス生成取得
+
+=head4 C<< getDB >>
+
+  $DB = $TL->getDB($group)
+
+L<Tripletail::DB> オブジェクトを取得。
+
+=head4 C<< newDB >>
+
+  $DB = $TL->newDB($group)
+
+L<Tripletail::DB> オブジェクトを作成。
+
+=head4 C<< newForm >>
+
+L<Tripletail::Form> オブジェクトを作成。
+
+=head4 C<< newTemplate >>
+
+L<Tripletail::Template> オブジェクトを作成。
+
+=head4 C<< getSession >>
+
+L<Tripletail::Session> オブジェクトを取得。
+
+=head4 C<< newValidator >>
+
+L<Tripletail::Validator> オブジェクトを生成。
+
+=head4 C<< newValue >>
+
+L<Tripletail::Value> オブジェクトを作成。
+
+=head4 C<< newDateTime >>
+
+L<Tripletail::DateTime> オブジェクトを作成。
+
+=head4 C<< newPager >>
+
+L<Tripletail::Pager> オブジェクトを作成。
+
+=head4 C<< getCsv >>
+
+L<Tripletail::CSV> オブジェクトを取得。
+
+=head4 C<< newTagCheck >>
+
+L<Tripletail::TagCheck> オブジェクトを作成。
+
+=head4 C<< newHtmlFilter >>
+
+L<Tripletail::HtmlFilter> オブジェクトを作成。
+
+=head4 C<< newHtmlMail >>
+
+L<Tripletail::HtmlMail> オブジェクトを作成。
+
+=head4 C<< newMail >>
+
+L<Tripletail::Mail> オブジェクトを作成。
+
+=head4 C<< newIni >>
+
+L<Tripletail::Ini> オブジェクトを作成。
+
+=head4 C<< getCookie >>
+
+L<Tripletail::Cookie> オブジェクトを取得。
+
+=head4 C<< getRawCookie >>
+
+L<Tripletail::RawCookie> オブジェクトを取得。
+
+=head4 C<< newSendmail >>
+
+L<Tripletail::Sendmail> オブジェクトを作成。
+
+=head4 C<< newSMIME >>
+
+L<Tripletail::SMIME> オブジェクトを作成。
+
+=head4 C<< getFileSentinel >>
+
+L<Tripletail::FileSentinel> オブジェクトを取得。
+
+=head4 C<< getMemorySentinel >>
+
+L<Tripletail::MemorySentinel> オブジェクトを取得。
+
+=head4 C<< newMemCached >>
+
+L<Tripletail::MemCached> オブジェクトを生成。
+
+=head3 その他
+
+=head4 C<< INI >>
+
+  $TL->INI
+
+C<< use Tripletail qw(filename.ini); >> で読み込まれた L<Tripletail::Ini> を返す。
+
+=head4 C<< trapError >>
+
+  $TL->trapError(
+    -main => \&Main, # メイン関数
+    -DB   => 'DB',   # DBを使う場合，iniのグループ名を指定
+  );
+
+環境を整え、 L</"Main関数"> を実行する。
+L</"Main関数"> がdieした場合は、エラー内容が標準エラーへ出力される。
+
+L</"startCgi"> と同様に、C<DB> には配列へのリファレンスを渡す事も出来る。
+
+=head4 C<< log >>
+
+  $TL->log($group => $log)
+
+ログを記録する。グループとログデータの２つを受け取る。
+
+第一引数のグループは省略可能。
+
+ログにはヘッダが付けられ、ヘッダは「時刻(epoch値の16進数8桁表現) プロセスIDの16進数4桁表現 FastCGIのリクエスト回数の16進数4桁表現 [グループ]」の形で付けられる。
+
+=head4 C<< setContentFilter >>
+
+  $TL->setContentFilter($classname, %option)
+  $TL->setContentFilter([$classname, $priority], %option)
+  $TL->setContentFilter('Tripletail::Filter::HTML', charset => 'Shift_JIS')
+  $TL->setContentFilter(
+      'Tripletail::Filter::CSV', charset => 'Shift_JIS', filename => 'テストデータ.csv')
+
+L</"出力フィルタ"> を設定する。
+全ての出力の前に実行する必要がある。
+２番目の書式では、プライオリティを指定して独自のコンテンツフィルタを
+追加できる。省略時は優先度は1000となる。小さい優先度のフィルタが先に、
+大きい優先度のフィルタが後に呼ばれる。同一優先度のフィルタが既に
+セットされているときは、以前のフィルタ設定は解除される。
+
+返される値は、指定された L<Tripletail::Filter> のサブクラスのインスタンスである。
+
+設定したフィルタは、L</"preRequest"> のタイミングで保存され、
+L</"postRequest"> のタイミングで元に戻される。従って、L</"Main関数">内
+で setContentFilter を実行した場合、その変更は次回リクエスト時に持ち越
+されない。
+
+=head4 C<< getContentFilter >>
+
+  $TL->getContentFilter($priority)
+
+指定されたプライオリティのフィルタを取得する。省略時は1000となる。
+
+=head4 C<< removeContentFilter >>
+
+  $TL->removeContentFilter($priority)
+
+指定されたプライオリティのフィルタを削除する。省略時は1000となる。
+フィルタが１つもない場合は、致命的エラーとなり出力関数は使用できなくなる。
+
+=head4 C<< getLogHeader >>
+
+  my $logid = $TL->getLogHeader
+
+ログを記録するときのヘッダと同じ形式の文字列を生成する。
+「時刻(epoch値の16進数8桁表現) プロセスIDの16進数4桁表現 FastCGIのリクエスト回数の16進数4桁表現」の形の文字列が返される。
+
+=head4 C<< setHook >>
+
+  $TL->setHook($type, $priority, \&func)
+
+指定タイプの指定プライオリティのフックを設定する。
+既に同一タイプで同一プライオリティのフックが設定されていた場合、
+古いフックの設定は解除される。
+
+typeは、L</"init">, L</"term">, L</"preRequest">, L</"postRequest">
+の４種類が存在する。
+
+なお、1万の整数倍のプライオリティは Tripletail 内部で使用される。アプリ
+ケーション側で不用意に用いるとフックを上書きしてしまう可能性があるので
+注意する。
+
+=head4 C<< removeHook >>
+
+  $TL->removeHook($type, $priority)
+
+指定タイプの指定プライオリティのフックを削除する。
+
+=head4 C<< setInputFilter >>
+
+  $TL->setInputFilter($classname, %option)
+  $TL->setInputFilter([$classname, $priority], %option)
+
+L</"入力フィルタ"> を設定する。
+L</"startCgi"> の前に実行する必要がある。
+
+返される値は、指定された L<Tripletail::InputFilter> のサブクラスのインスタンスである。
+
+=head4 C<< getInputFilter >>
+
+  $TL->getInputFilter($priority)
+
+=head4 C<< removeInputFilter >>
+
+  $TL->removeInputFilter($priority)
+
+=head4 C<< sendError >>
+
+  $TL->sendError(-title => "タイトル", -error => "エラー")
+
+L<ini|Tripletail::Ini> で指定されたアドレスにエラーメールを送る。
+設定が無い場合は何もしない。
+
+=head4 C<< readFile >>
 
   $data = $TL->readFile($fpath);
 
 ファイルを読み込む。文字コード変換をしない。
 ファイルロック処理は行わないので、使用の際には注意が必要。
 
-=item C<< readTextFile >>
+=head4 C<< readTextFile >>
 
   $data = $TL->readTextFile($fpath, $coding, $prefer_encode);
 
@@ -2417,7 +2579,7 @@ C<$to> が省略された場合は C<'UTF-8'> になる。
 C<$coding> が省略された場合は C<'auto'> となる。C<$prefer_encode> を真
 にすると L<Encode> が利用可能な場合は L<Encode> で変換する。
 
-=item C<< writeFile >>
+=head4 C<< writeFile >>
 
   $TL->writeFile($fpath, $fdata, $fmode);
 
@@ -2429,7 +2591,7 @@ C<$fmode> が1ならば、追加モード。
 
 省略された場合は上書きモードとなる。
 
-=item C<< writeTextFile >>
+=head4 C<< writeTextFile >>
 
   $TL->writeTextFile($fpath, $fdata, $fmode, $coding, $prefer_encode);
 
@@ -2444,7 +2606,7 @@ C<$fmode> が1ならば、追加モード。
 C<$coding> が省略された場合、utf8として扱う。C<$prefer_encode> を真にする
 と L<Encode> が利用可能な場合は L<Encode> で変換する。
 
-=item C<< watch >>
+=head4 C<< watch >>
 
   $TL->watch(sdata => \$sdata, $reclevel);
   $TL->watch(adata => \@adata, $reclevel);
@@ -2460,18 +2622,22 @@ C<$coding> が省略された場合、utf8として扱う。C<$prefer_encode> 
 
 また、再帰的にウォッチする場合、変数名は親の変数名を利用して自動的に設定される。
 
-=item C<< dump >>
+=head4 C<< dump >>
 
+  $TL->dump(\$data);
+  $TL->dump(\$data, $level);
   $TL->dump(DATA => \$data);
   $TL->dump(DATA => \$data, $level);
 
 第2引数に変数へのリファレンスを渡すと，その内容を Data::Dumper でダンプし、
 第1引数のグループ名で $TL->log を呼び出す。
 
+第1引数のグループ名は省略可能。
+
 第3引数で、リファレンスをどのくらいの深さまで追うかを指定することが出来る。
 指定しなければ全て表示される。
 
-=item C<< printCacheUnlessModified >>
+=head4 C<< printCacheUnlessModified >>
 
   $bool = $TL->printCacheUnlessModified($key, $flag, $param, $charset)
 
@@ -2505,7 +2671,7 @@ UTF-8，Shift_JIS，EUC-JP，ISO-2022-JP
 
 この関数からundefを返された場合、以後出力を行う操作を行ってはならない。
 
-=item C<< setCache >>
+=head4 C<< setCache >>
 
   $TL->setCache($key, $param, $charset, $priority)
 
@@ -2532,7 +2698,7 @@ UTF-8，Shift_JIS，EUC-JP，ISO-2022-JP
 Tripletail::Filter::MemCachedは必ず最後に実行する必要性があるため、
 1500以上の優先度で設定するフィルタが他にある場合は手動で設定する必要がある。
 
-=item C<< deleteCache >>
+=head4 C<< deleteCache >>
 
   $TL->deleteCache($key)
 
@@ -2546,15 +2712,13 @@ Tripletail::Filter::MemCachedは必ず最後に実行する必要性があるた
 該当するキャッシュを削除するように使用する。
 それにより、次回アクセス時に最新の情報が出力される。
 
-=item C<< getDebug >>
+=head4 C<< getDebug >>
 
-=item C<< newError >>
+=head4 C<< newError >>
 
 内部用メソッド。
 
 
-=back
-
 =head2 Ini パラメータ
 
 グループ名は常に B<TL> でなければならない。
@@ -2736,6 +2900,7 @@ startCgi メソッド中で出力をバッファリングする。デフォル
 
 各種タイムアウト時間，セッションのexpiresなど、
 時間間隔は以下の指定が可能とする。
+数値化には L</parsePeriod> を使用する. 
 
 単位は大文字小文字を区別しない。
 
@@ -2763,7 +2928,11 @@ startCgi メソッド中で出力をバッファリングする。デフォル
 
 =item 数値＋ 'mon' or 'month' or 'months'
 
-月での指定を表す。１月＝30日として計算する。 [×30*24*3600]
+月での指定を表す。１月＝30.436875日として計算する。 [×30.436875*24*3600]
+
+=item 数値＋ 'year' or 'years'
+
+年での指定を表す。１年＝365.2425日として計算する。 [×365.2425*24*3600]
 
 =back
 
@@ -2772,6 +2941,8 @@ startCgi メソッド中で出力をバッファリングする。デフォル
 メモリサイズ、文字列サイズ等、大きさを指定する場合には、
 以下の指定が可能とする。英字の大文字小文字は同一視する。
 
+数値化には L</parseQuantity> を使用する. 
+
 =head4 10進数系
 
 =over 4
@@ -2836,6 +3007,112 @@ startCgi メソッド中で出力をバッファリングする。デフォル
 
 =back
 
+=head1 SAMPLE
+
+ perldoc -u Tripletail |  podselect -sections SAMPLE | sed -e '1,4d' -e 's/^ //'
+
+
+ # master configurations.
+ #
+ [TL]
+ logdir=/home/project/logs/error
+ errormail=errors@your.address
+ errorlog=2
+ trap=diewithprint
+ stackallow=0.0.0.0/0
+ 
+ [TL:SmallDebug]
+ stacktrace=onlystack
+ outputbuffering=1
+ 
+ [TL:Debug]
+ stacktrace=full
+ outputbuffering=1
+ 
+ [TL:FullDebug]
+ stacktrace=full
+ outputbuffering=1
+ stackallow=0.0.0.0/0
+ 
+ # database configrations.
+ #
+ [DB]
+ type=mysql
+ namequery=1
+ tracelevel=0
+ AllTransaction=DBALL
+ defaultset=AllTransaction
+ [DBALL]
+ dbname=
+ user=
+ password=
+ #host=
+ 
+ [DB:SmallDebug]
+ 
+ [DB:Debug]
+ 
+ [DB:FullDebug]
+ tracelevel=2
+ 
+ # debug configrations.
+ #
+ [Debug]
+ enable_debug=0
+ 
+ [Debug:SmallDebug]
+ enable_debug=1
+ 
+ [Debug:Debug]
+ enable_debug=1
+ request_logging=1
+ content_logging=1
+ warn_logging=1
+ db_profile=1
+ popup_type=single
+ template_popup=0
+ request_popup=1
+ db_popup=0
+ log_popup=1
+ warn_popup=1
+ 
+ [Debug:FullDebug]
+ enable_debug=1
+ request_logging=1
+ content_logging=0
+ warn_logging=1
+ db_profile=1
+ popup_type=single
+ template_popup=1
+ request_popup=1
+ db_popup=1
+ log_popup=1
+ warn_popup=1
+ location_debug=1
+ 
+ # misc.
+ #
+ [SecureCookie]
+ path=/
+ secure=1
+ 
+ [Session]
+ mode=https
+ securecookie=SecureCookie
+ timeout=30min
+ updateinterval=10min
+ dbgroup=DB
+ dbset=AllTransaction
+ 
+ # user data.
+ # you can read this data:
+ # $val = $TL->INI->get(UserData=>'roses');
+ #
+ [UserData]
+ roses=red
+ violets=blue
+ sugar=sweet
+
 =head1 SEE ALSO
 
 =over 4
