@@ -6,13 +6,18 @@ use strict;
 use warnings;
 use Tripletail;
 require Time::HiRes;
+use DBI qw(:sql_types);
 
-sub _PRE_REQUEST_HOOK_PRIORITY() { -1_000_000 } # 順序は問わない
+sub _PRE_REQUEST_HOOK_PRIORITY()  { -1_000_000 } # 順序は問わない
 sub _POST_REQUEST_HOOK_PRIORITY() { -1_000_000 } # セッションフックの後
-sub _TERM_HOOK_PRIORITY() { -1_000_000 } # セッションフックの後
+sub _TERM_HOOK_PRIORITY()         { -1_000_000 } # セッションフックの後
 
 our $INSTANCES = {}; # グループ名 => インスタンス
 
+sub _TX_STATE_NONE()      { 0 }
+sub _TX_STATE_ACTIVE()    { 1 }
+sub _TX_STATE_CLOSEWAIT() { 2 }
+
 1;
 
 sub _getInstance {
@@ -27,8 +32,9 @@ sub _getInstance {
 
 	my $obj = $INSTANCES->{$group};
 	if(!$obj) {
-		die "TL#getDB, DB [$group] was not specified in the startCgi() / errorTrap().\n";
+		die "TL#getDB, DB [$group] was not specified in the startCgi() / trapError().\n";
 	}
+	$obj->{tx_state}!=_TX_STATE_NONE and die "tx state : $obj->{tx_state}";
 
 	$obj;
 }
@@ -42,6 +48,7 @@ sub connect {
 			$dbh->connect($this->{type});
 		}
 	}
+	$this->{tx_state} = _TX_STATE_NONE;
 
 	$this;
 }
@@ -52,26 +59,102 @@ sub disconnect {
 	foreach my $dbh (values %{$this->{dbname}}) {
 		$dbh->disconnect;
 	}
+	$this->{tx_state} = _TX_STATE_NONE;
 
 	$this;
 }
 
+sub tx
+{
+	my $this = shift;
+	my $setname = !ref($_[0]) && shift;
+	my $sub  = shift;
+	
+	require Sub::ScopeFinalizer;
+	my @ret;
+	
+	$this->{tx_state}==_TX_STATE_CLOSEWAIT and $this->_closewait_broken();
+	my $succ = 0;
+	my $anchor = Sub::ScopeFinalizer->new(sub{
+		$succ and return;
+		# on exception.
+		if( $this->{tx_state}==_TX_STATE_ACTIVE )
+		{
+			$this->rollback();
+		}
+		$this->{tx_state} = _TX_STATE_NONE;
+	});
+	$this->begin($setname);
+	$this->{tx_state} = _TX_STATE_ACTIVE;
+	if( wantarray )
+	{
+		@ret    = $sub->();
+	}else
+	{
+		$ret[0] = $sub->();
+	}
+	if( $this->{trans_dbh} )
+	{
+		$this->commit();
+	}
+	$this->{tx_state} = _TX_STATE_NONE;
+	$succ = 1;
+	wantarray ? @ret : $ret[0];
+}
+
+sub _closewait_broken
+{
+	my $this = shift;
+	my $where = shift;
+	if( !$where )
+	{
+		$where = (caller(1))[3];
+		$where =~ s/.*:://;
+	}
+	die __PACKAGE__."#$where, act something on close-wait transaction";
+}
+sub inTx
+{
+	my $this = shift;
+	my $set  = shift;
+	$this->_requireTx($set, 'inTx');
+}
+sub _requireTx
+{
+	my $this    = shift;
+	my $setname = shift;
+	my $where   = shift;
+	
+	$this->{tx_state}==_TX_STATE_CLOSEWAIT and $this->_closewait_broken($where);
+	if( my $trans = $this->{trans_dbh} )
+	{
+		my $set = $this->_getDbSetName($setname);
+		
+		my $trans_set = $trans->getSetName;
+		if($trans_set eq $set)
+		{
+			# same transaction.
+			return 1;
+		}
+		# another transaction running, always die.
+		die __PACKAGE__."#$where, attempted to begin a".
+			" new transaction on DB Set [$set] but".
+			" another DB Set [$trans_set] were already in transaction.".
+			" Commit or rollback it before begin another one.\n";
+	}else
+	{
+		# no transaction.
+		return 0;
+	}
+}
+
 sub begin {
 	my $this = shift;
 	my $setname = shift;
 
 	my $set = $this->_getDbSetName($setname);
 
-	if(my $trans = $this->{trans_dbh}) {
-		my $trans_set = $trans->getSetName;
-		if($trans_set ne $set) {
-			die __PACKAGE__."#begin, attempted to begin a transaction on DB Set [$set] but".
-				" another DB Set [$trans_set] were already in transaction.".
-				" Commit or rollback it before begin another one.\n";
-		} else {
-			die __PACKAGE__."#begin, already in transaction.";
-		}
-	}
+	$this->_requireTx($setname, 'begin');
 
 	my $begintime = [Time::HiRes::gettimeofday()];
 
@@ -98,7 +181,8 @@ sub begin {
 
 sub rollback {
 	my $this = shift;
-
+	$this->{tx_state}==_TX_STATE_CLOSEWAIT and $this->_closewait_broken();
+	
 	my $dbh = $this->{trans_dbh};
 	if(!defined($dbh)) {
 		die __PACKAGE__."#rollback, not in transaction.\n";
@@ -107,6 +191,10 @@ sub rollback {
 	my $begintime = [Time::HiRes::gettimeofday()];
 
 	$dbh->rollback;
+	if( $this->{tx_state}==_TX_STATE_ACTIVE )
+	{
+		$this->{tx_state} = _TX_STATE_CLOSEWAIT;
+	}
 
 	my $elapsed = Time::HiRes::tv_interval($begintime);
 
@@ -128,6 +216,7 @@ sub rollback {
 
 sub commit {
 	my $this = shift;
+	$this->{tx_state}==_TX_STATE_CLOSEWAIT and $this->_closewait_broken();
 
 	my $dbh = $this->{trans_dbh};
 	if (!defined($dbh)) {
@@ -137,6 +226,10 @@ sub commit {
 	my $begintime = [Time::HiRes::gettimeofday()];
 
 	$dbh->commit;
+	if( $this->{tx_state}==_TX_STATE_ACTIVE )
+	{
+		$this->{tx_state} = _TX_STATE_CLOSEWAIT;
+	}
 
 	my $elapsed = Time::HiRes::tv_interval($begintime);
 
@@ -172,6 +265,8 @@ sub setDefaultSet {
 sub execute {
 	my $this = shift;
 	my $dbset = shift;
+	$this->{tx_state}==_TX_STATE_CLOSEWAIT and $this->_closewait_broken();
+	
 	if(ref($dbset)) {
 		$dbset = $$dbset;
 	} else {
@@ -244,7 +339,7 @@ sub execute {
 			die __PACKAGE__."#execute, use `??' instead of `?' if you want to use ARRAY Ref as a bind parameter.\n";
 		}
 	}
-
+	
 	# executeを行うDBセットを探す
 	my $dbh = undef;
 	if(defined($dbset)) {
@@ -258,12 +353,23 @@ sub execute {
 		$dbh = $this->{locked_dbh} if(!$dbh);
 		$dbh = $this->{dbh}{$this->_getDbSetName} if(!$dbh);
 	}
+	
+	if( $dbh->{bindconvert} )
+	{
+		my $sub = $dbh->{bindconvert};
+		$dbh->$sub(\$sql, \@params);
+	}
 
 	my $sth = Tripletail::DB::STH->new(
 		$this,
 		$dbh,
 		$dbh->getDbh->prepare($sql)
 	);
+	if( $dbh->{fetchconvert} )
+	{
+		my $sub = $dbh->{fetchconvert};
+		$dbh->$sub($sth, new => [\$sql, \@params]);
+	}
 
 	# 全てのパラメータをbind_paramする。
 	for(my $i = 0; $i < @params; $i++) {
@@ -339,8 +445,9 @@ sub execute {
 
 sub selectAllHash {
 	my $this = shift;
+	$this->{tx_state}==_TX_STATE_CLOSEWAIT and $this->_closewait_broken();
+	
 	my $sth = $this->execute(@_);
-
 	my $result = [];
 	while(my $data = $sth->fetchHash) {
 		push @$result, { %$data };
@@ -350,8 +457,9 @@ sub selectAllHash {
 
 sub selectAllArray {
 	my $this = shift;
-	my $sth = $this->execute(@_);
+	$this->{tx_state}==_TX_STATE_CLOSEWAIT and $this->_closewait_broken();
 
+	my $sth = $this->execute(@_);
 	my $result = [];
 	while (my $data = $sth->fetchArray) {
 		push @$result, [ @$data ];
@@ -361,8 +469,9 @@ sub selectAllArray {
 
 sub selectRowHash {
 	my $this = shift;
-	my $sth = $this->execute(@_);
+	$this->{tx_state}==_TX_STATE_CLOSEWAIT and $this->_closewait_broken();
 	
+	my $sth = $this->execute(@_);
 	my $data = $sth->fetchHash();
 	$data = $data ? {%$data} : undef;
 	$sth->finish();
@@ -372,9 +481,9 @@ sub selectRowHash {
 
 sub selectRowArray {
 	my $this = shift;
+	$this->{tx_state}==_TX_STATE_CLOSEWAIT and $this->_closewait_broken();
+	
 	my $sth = $this->execute(@_);
-	
-	
 	my $data = $sth->fetchArray();
 	$data = $data ? [@$data] : undef;
 	$sth->finish();
@@ -463,6 +572,7 @@ sub lock {
 
 sub unlock {
 	my $this = shift;
+	$this->{tx_state}==_TX_STATE_CLOSEWAIT and $this->_closewait_broken();
 
 	my $dbh = $this->{locked_dbh};
 	if(!defined($dbh)) {
@@ -502,6 +612,15 @@ sub setBufferSize {
 	$this;
 }
 
+sub getLastInsertId
+{
+	my $this = shift;
+	my $dbh = $this->{trans_dbh};
+	$dbh ||= $this->{locked_dbh};
+	$dbh ||= $this->{dbh}{$this->_getDbSetName};
+	$dbh->getLastInsertId();
+}
+
 sub symquote {
 	my $this = shift;
 	my $str = shift;
@@ -563,13 +682,13 @@ sub _getDbSetName {
 }
 
 sub _connect {
-	# クラスメソッド。TL#startCgi，TL#errorTrapのみが呼ぶ。
+	# クラスメソッド。TL#startCgi，TL#trapErrorのみが呼ぶ。
 	my $class = shift;
 	my $groups = shift;
 
 	foreach my $group (@$groups) {
 		if (!defined($group)) {
-			die "TL#startCgi, -DB had an unref value.\n";
+			die "TL#startCgi, -DB had an undef value.\n";
 		} elsif(ref($group)) {
 			die "TL#startCgi, -DB had a Ref.\n";
 		}
@@ -616,8 +735,9 @@ sub _new {
 
 	$this->{default_set} = $TL->INI->get($group => 'defaultset', undef); # デフォルトのセット名
 
-	$this->{locked_dbh} = undef; # Tripletail::DB::Dbh
-	$this->{trans_dbh} = undef; # Tripletail::DB::Dbh
+	$this->{locked_dbh}   = undef; # Tripletail::DB::Dbh
+	$this->{trans_dbh}    = undef; # Tripletail::DB::Dbh
+	$this->{tx_state}     = _TX_STATE_NONE;
 
 	do {
 		local $SIG{__DIE__} = 'DEFAULT';
@@ -739,6 +859,7 @@ sub new {
 	$this->{setname} = $setname;
 	$this->{inigroup} = $dbname;
 	$this->{dbh} = undef; # DBI-dbh
+	$this->{type} = undef; # set on connect().
 
 	$this;
 }
@@ -772,6 +893,7 @@ sub connect {
 	my $type = shift;
 
 	$type or die __PACKAGE__."#connect, type is not set.\n";
+	$this->{type} = $type;
 	if($type eq 'mysql') {
 		my $opts = {
 			dbname => $TL->INI->get($this->{inigroup} => 'dbname'),
@@ -873,6 +995,129 @@ sub connect {
 			PrintError => 0,
 			RaiseError => 1,
 		});
+	} elsif($type eq 'sqlite') {
+		my $opts = {
+			dbname => $TL->INI->get($this->{inigroup} => 'dbname'),
+		};
+		$opts->{dbname} or die __PACKAGE__."#connect, dbname is not set.\n";
+
+		$this->{dbh} = DBI->connect(
+			'dbi:SQLite:' . join(';', map { "$_=$opts->{$_}" } keys %$opts),
+			$TL->INI->get($this->{inigroup} => 'user'),
+			$TL->INI->get($this->{inigroup} => 'password'), {
+			AutoCommit => 1,
+			PrintError => 0,
+			RaiseError => 1,
+		});
+	} elsif($type eq 'mssql' || $type eq 'odbc' ) {
+		my $nl = $Tripletail::_CHKNONLAZY || 0;
+		my $dl = $Tripletail::_CHKDYNALDR || 0;
+		my $opts = {
+			map{ $_ => $TL->INI->get($this->{inigroup} => $_) }
+				qw(dbname host port tdsname odbcdsn odbcdriver),
+				qw(bindconvert fetchconvert)
+		};
+		$opts->{dbname} or die __PACKAGE__."#connect, dbname is not set.\n";
+
+		# build data source string.
+		my $dsn;
+		if( $opts->{odbcdsn} )
+		{
+			my $odbcdsn = $opts->{odbcdsn} || '';
+			if( $odbcdsn =~ m/^(\w+)$/ )
+			{
+				$dsn = "dbi:ODBC:DSN=$odbcdsn";
+			}elsif( $odbcdsn =~ /^Driver=|^DSN=/i )
+			{
+				$dsn = "dbi:ODBC:$odbcdsn";
+			}elsif( $odbcdsn =~ /^dbi:/ )
+			{
+				$dsn = $odbcdsn;
+			}else
+			{
+				die __PACKAGE__."#connect, unknown odbcdsn.\n";
+			}
+		}
+		# odbc driver.
+		if( !$dsn || $dsn!~/[:;]Driver=/i || $opts->{odbcdriver} )
+		{
+			my $driver = $opts->{odbcdriver};
+			$driver ||= $^O eq 'MSWin32' ? 'SQL Server' : 'freetdsdriver';
+			if( !$dsn )
+			{
+				$dsn = "dbi:ODBC:DRIVER={$driver}";
+			}else
+			{
+				$dsn .= ";DRIVER={$driver}";
+			}
+		}
+		$opts->{tdsname} and $dsn .= ";Servername=$opts->{tdsname}";
+		$opts->{host}    and $dsn .= ";Server=$opts->{host}";
+		$opts->{port}    and $dsn .= ";Port=$opts->{port}";
+		$opts->{dbname}  and $dsn .= ";Database=$opts->{dbname}";
+		
+		# bindconvert/fetchconvert from driver.
+		my $odbc_driver = $dsn =~ /[:;]DRIVER=\{(.*?)\}/i ? lc($1) : '';
+		if( $odbc_driver eq 'freetdsdriver' )
+		{
+			$opts->{bindconvert} ||= 'freetds';
+		}elsif( $odbc_driver eq 'sql server' )
+		{
+			local($SIG{__DIE__},$@) = 'DEFAULT';
+			my $codepage = eval{
+				require Win32::API;
+				my $get_acp   = Win32::API->new("kernel32", "GetACP",   "", "N");
+				$get_acp && $get_acp->Call();
+			} || 0;
+			if( !$codepage || $codepage==932 )
+			{
+				$opts->{bindconvert}  ||= 'mssql_cp932';
+				$opts->{fetchconvert} ||= 'mssql_cp932';
+				#$dsn .= ';AutoTranslate=No';
+			}
+		}
+		foreach my $key (qw(bindconvert fetchconvert))
+		{
+			$opts->{$key} or next;
+			$opts->{$key} eq 'no' and next;
+			my $sub = $this->can("_${key}_$opts->{$key}");
+			$sub or die __PACKAGE__."#connect, no such $key: $opts->{$key}";
+			$this->{$key} = $sub;
+		}
+		
+		#print "dsn = [$dsn]\n";
+		my $conn = sub{
+			$this->{dbh} = DBI->connect(
+				$dsn,
+				$TL->INI->get($this->{inigroup} => 'user'),
+				$TL->INI->get($this->{inigroup} => 'password'),
+				{
+					AutoCommit => 1,
+					PrintError => 0,
+					RaiseError => 1,
+				}
+			);
+		};
+		if( (!$dl && $nl) || $^O eq 'MSWin32' )
+		{
+			$conn->();
+		}else
+		{
+			eval{ $conn->(); };
+			if( $@ )
+			{
+				my $err = $@;
+				chomp $err;
+				$err .= " (perhaps you forgot to set env PERL_DL_NONLAZY=1?)";
+				$err .= " ...propagated";
+				die $err;
+			}
+		}
+		if( $this->{fetchconvert} )
+		{
+			my $sub = $this->{fetchconvert};
+			$this->$sub(undef, connect => undef);
+		}
 	} else {
 		die __PACKAGE__."#connect, DB type [$type] is not supported.\n";
 	}
@@ -884,6 +1129,197 @@ sub connect {
 	$this;
 }
 
+sub getLastInsertId
+{
+	my $this = shift;
+	my $type = $this->{type} or die __PACKAGE__."#getLastInsertId(), no type set";
+	my $obj  = shift; # for sequence on pgsql and oracle?
+	if( $type eq 'mysql' )
+	{
+		return $this->{dbh}{mysql_insertid};
+	}elsif($type eq 'pgsql')
+	{
+		my ($curval) = $this->{dbh}->selectrow_array(q{
+			SELECT currval(?)
+		}, $obj);
+		return $curval;
+	}elsif($type eq 'oracle')
+	{
+		$obj =~ /^((?:\w+\.)\w+)$/ or die __PACKAGE__."#getLastInsertId(), TODO: symquote for oracle is under construction";
+  		my $obj_sym = $1;
+		my ($curval) = $this->{dbh}->selectrow_array(q{
+			SELECT $obj_sym.curval
+			  FROM dual
+		});
+		return $curval;
+	}elsif( $type eq 'sqlite' )
+	{
+		return $this->{dbh}->func('last_insert_rowid');
+	}elsif( $type eq 'mssql' )
+	{
+		my ($curval) = $this->{dbh}->selectrow_array(q{
+			SELECT @@IDENTITY
+		});
+		return $curval;
+	}else
+	{
+		die __PACKAGE__."#getLastInsertId(), $type is not supported.";
+	}
+}
+
+sub _bindconvert_freetds
+{
+	my $this = shift;
+	my $ref_sql = shift;
+	my $params  = shift;
+	my $i = -1;
+	foreach my $elm (@$params)
+	{
+		++$i;
+		ref($elm) or next;
+		if( ${$elm->[1]} eq 'SQL_WVARCHAR' )
+		{
+			my $u = $TL->charconv($elm->[0], 'utf8', 'ucs2');
+			$elm->[0] = pack("v*",unpack("n*",$u));
+			$elm->[1] = \'SQL_BINARY';
+			
+			my $l = length($u)/2;
+			my $j = 0;
+			my $repl = "CAST(? AS NVARCHAR($l))";
+			$$ref_sql =~ s{\?}{$j++==$i?$repl:'?'}ge;
+		}
+	}
+}
+
+sub _bindconvert_mssql_cp932
+{
+	my $this = shift;
+	my $ref_sql = shift;
+	my $params  = shift;
+	$$ref_sql = $TL->charconv($$ref_sql, 'utf8' => 'sjis');
+	my $i = -1;
+	foreach my $elm (@$params)
+	{
+		++$i;
+		if( !ref($elm) )
+		{
+			$elm = $TL->charconv($elm, 'utf8', 'sjis');
+		}elsif( ${$elm->[1]} =~ /^SQL_W(?:(?:LONG)?VAR)?CHAR$/ )
+		{
+			my $u = $TL->charconv($elm->[0], 'utf8', 'ucs2');
+			$elm->[0] = pack("v*",unpack("n*",$u));
+			$elm->[1] = \'SQL_BINARY';
+			
+			my $l = length($u)/2;
+			my $j = 0;
+			my $repl = "CAST(? AS NVARCHAR($l))";
+			$$ref_sql =~ s{\?}{$j++==$i?$repl:'?'}ge;
+		}elsif( ${$elm->[1]} =~ /^SQL_(?:(?:LONG)?VAR)?CHAR$/ )
+		{
+			$elm = $TL->charconv($elm, 'utf8', 'sjis');
+		}
+	}
+}
+sub _fetchconvert
+{
+	my $this = shift;
+	if( $this->{fetchconvert} )
+	{
+		my $sub = $this->{fetchconvert};
+		$this->$sub(@_);
+	}
+}
+sub _fetchconvert_mssql_cp932
+{
+	my $this = shift;
+	my $sth  = shift;
+	my $mode = shift;
+	my $obj  = shift;
+	
+	if( $mode eq 'new' )
+	{
+		# obj is [\$sql, \@params];
+		
+		# なんだか先に一回やっとかないとおかしくなる?
+		$this->{dbh}->type_info(1);
+		
+		my $types = $sth->{sth}{TYPE};
+		my $dbh = $sth->{dbh}{dbh};
+		my @types = map{ $dbh->type_info($_)->{TYPE_NAME} } @$types;
+		$sth->{_types} = \@types;
+		$sth->{_name_hash} = {%{$sth->{sth}{NAME_hash}||{}}}; # raw encoded.
+		my @names;
+		while(my($k,$v)=each%{$sth->{_name_hash}})
+		{
+			$names[$v] = $TL->charconv($k, "sjis" => "utf8");
+		}
+		$sth->{_name_arraymap} = \@names;
+		$sth->{_decode_cols} = [];
+	}elsif( $mode eq 'nameArray' )
+	{
+		# obj is arrayref.;
+		@$obj = @{$sth->{sth}{NAME}||[]};
+		foreach my $elm (@$obj)
+		{
+			$elm = lc $TL->charconv($elm, 'cp932' => 'utf8');
+		}
+	}elsif( $mode eq 'nameHash' )
+	{
+		# obj is hashref.
+		foreach my $key (keys %$obj)
+		{
+			my $ukey = lc $TL->charconv($key, 'cp932' => 'utf8');
+			$obj->{$ukey} = delete $obj->{$key};
+		}
+	}elsif( $mode eq 'fetchArray' )
+	{
+		# obj is arrayref.
+		my $i = -1;
+		foreach my $val (@$obj)
+		{
+			++$i;
+			defined($val) or next;
+			my $type = (defined($i) && $sth->{_types}[$i]) || '';
+			if( $type =~ /^n?((long)?var)?char$/ )
+			{
+				if( defined($val) && $val =~ /[^\0-\x7f]/ )
+				{
+					$val = $TL->charconv($val, 'cp932' => 'utf8');
+				}
+			}
+			my $ukey = $sth->{_name_arraymap}[$i] || "\0";
+			if( grep{$_ eq $i || $_ eq $ukey} @{$sth->{_decode_cols}} )
+			{
+				my $bin = pack("v*",unpack("n*",$val));
+				$val = $TL->charconv($bin,"ucs2","utf8");
+			}
+		}
+	}elsif( $mode eq 'fetchHash' )
+	{
+		# obj is hashref.
+		foreach my $key (keys %$obj)
+		{
+			my $ukey = $TL->charconv($key, 'cp932' => 'utf8');
+			my $i = $sth->{_name_hash}{$key}; # raw encoded.
+			my $type = (defined($i) && $sth->{_types}[$i]) || '*';
+			my $val = delete $obj->{$key};
+			if( $type =~ /^n?((long)?var)?char$/ )
+			{
+				if( defined($val) && $val =~ /[^\0-\x7f]/ )
+				{
+					$val = $TL->charconv($val, 'cp932' => 'utf8');
+				}
+			}
+			if( grep{$_ eq $i || $_ eq $ukey} @{$sth->{_decode_cols}} )
+			{
+				my $bin = pack("v*",unpack("n*",$val));
+				$val = $TL->charconv($bin,"ucs2","utf8");
+			}
+			$obj->{$ukey} = $val;
+		}
+	}
+}
+
 sub disconnect {
 	my $this = shift;
 
@@ -945,6 +1381,11 @@ sub fetchHash {
 			data    => $hash,
 		);
 	}
+	if( $this->{dbh}{fetchconvert} )
+	{
+		my $sub = $this->{dbh}{fetchconvert};
+		$this->{dbh}->$sub($this, fetchHash => $hash);
+	}
 
 
 	if(my $lim = $this->{db_center}{bufsize}) {
@@ -965,6 +1406,11 @@ sub fetchArray {
 	my $this = shift;
 	my $array = $this->{sth}->fetchrow_arrayref;
 
+	if( $this->{dbh}{fetchconvert} )
+	{
+		my $sub = $this->{dbh}{fetchconvert};
+		$this->{dbh}->$sub($this, fetchArray => $array);
+	}
 	if($array) {
 		$TL->getDebug->_dbLogData(
 			group   => $this->{group},
@@ -1006,12 +1452,36 @@ sub finish {
 
 sub nameArray {
 	my $this = shift;
-	$this->{sth}{NAME_lc};
+	my $name_lc = $this->{sth}{NAME_lc};
+	if( $name_lc && $this->{dbh}{fetchconvert} )
+	{
+		$name_lc = [@{$this->{sth}{NAME}}]; # start from mixed case.
+		my $sub = $this->{dbh}{fetchconvert};
+		$this->{dbh}->$sub($this, nameArray => $name_lc);
+	}
+	$name_lc;
 }
 
 sub nameHash {
 	my $this = shift;
-	$this->{sth}{NAME_lc_hash};
+	my $name_lc_hash = $this->{sth}{NAME_lc_hash};
+	if( $name_lc_hash && $this->{dbh}{fetchconvert} )
+	{
+		$name_lc_hash = {%{$this->{sth}{NAME_hash}}}; # start from mixed case.
+		my $sub = $this->{dbh}{fetchconvert};
+		$this->{dbh}->$sub($this, nameHash => $name_lc_hash);
+	}
+	$name_lc_hash;
+}
+
+sub _fetchconvert
+{
+	my $this = shift;
+	if( $this->{dbh}{fetchconvert} )
+	{
+		my $sub = $this->{dbh}{fetchconvert};
+		$this->{dbh}->$sub($this, @_);
+	}
 }
 
 __END__
@@ -1028,22 +1498,23 @@ Tripletail::DB - DBIのラッパ
       -DB      => 'DB',
       -main        => \&main,
   );
-
+  
   sub main {
     my $DB = $TL->getDB('DB');
-
+    
     $DB->setDefaultSet('R_Trans');
-    $DB->begin;
-
-    my $sth = $DB->execute(q{SELECT a, b FROM foo WHERE a = ?}, 999);
-    while (my $hash = $sth->fetchHash) {
+    $DB->tx(sub{
+      my $sth = $DB->execute(q{SELECT a, b FROM foo WHERE a = ?}, 999);
+      while (my $hash = $sth->fetchHash) {
         print $hash->{a};
+      }
+      # commit is done implicitly.
+    });
+    
+    $DB->tx('W_Trans' => sub{
+      $DB->execute(q{UPDATE counter SET counter = counter + 1 WHERE id = ?}, 1);
+      $DB->commit; # can commit explicitly.
     }
-    $DB->commit;
-
-    $DB->begin('W_Trans');
-    $DB->execute(q{UPDATE counter SET counter = counter + 1 WHERE id = ?}, 1);
-    $DB->commit;
   }
 
 =head1 DESCRIPTION
@@ -1100,6 +1571,95 @@ DBコネクションの名称はCON_XXXX(XXXXは任意の文字列)でなけれ
 
 =back
 
+=head2 DBIからの移行
+
+TripletailのDBクラスはDBIに対するラッパの形となっており、多くのインタフェースはDBIのものとは異なる。
+ただし、いつでも C<< $DB->getDbh() >> メソッドにより元のDBIオブジェクトを取得できるので、DBIのインタフェースで利用することも可能となっている。
+
+DBIのインタフェースは以下のようなケースで利用できる。ただし、DBIを直接利用する場合は、TLの拡張プレースホルダやデバッグ機能、トランザクション整合性の管理などの機能は利用できない。
+
+=over 4
+
+=item ラッパに同等の機能が用意されていない場合。
+
+（例）$DB->{mysql_insertid}
+
+=item 高速な処理が必要で、ラッパのオーバヘッドを回避したい場合。
+
+DBIに対するラッパであるため、大量のSQLを実行する場合などはパフォーマンス上のデメリットがある。
+
+=back
+
+DBIでのSELECTは、以下のように置き換えられる。
+
+ # DBI
+ my $sth = $DB->prepare(q{SELECT * FROM test WHERE id = ?});
+ $sth->execute($id);
+ while(my $data = $sth->fetchrow_hashref) {
+ }
+ # TL
+ my $sth = $DB->execute(q{SELECT * FROM test WHERE id = ?}, $id);
+ while(my $data = $sth->fetchHash) {
+ }
+
+TLではprepare/executeは一括で行い、prepared statement は利用できない。
+
+INSERT・UPDATEは、以下のように置き換えられる。
+
+ # DBI
+ my $sth = $DB->prepare(q{INSERT INTO test VALUES (?, ?)});
+ my $ret = $sth->execute($id, $data);
+ # TL
+ my $sth = $DB->execute(q{INSERT INTO test VALUES (?, ?)}, $id, $data);
+ my $ret = $sth->ret;
+
+prepare/executeを一括で行うのは同様であるが、executeの戻り値は$sthオブジェクトであり、影響した行数を取得するためには C<< $sth->ret >> メソッドを呼ぶ必要がある。
+
+プレースホルダの型指定は以下のように行う。
+
+ # DBI
+ my $sth = $DB->prepare(q{SELECT * FROM test LIMIT ?});
+ $sth->bind_param(1, $limit, { TYPE => SQL_INTEGER });
+ $sth->execute;
+ # TL
+ my $sth = $DB->execute(q{SELECT * FROM test LIMIT ??}, [$limit, \'SQL_INTEGER']);
+
+TLの拡張プレースホルダ（??で表記される）を利用し、配列のリファレンスの最後に型をスカラのリファレンスの形で渡す。
+拡張プレースホルダでは、複数の値を渡すことも可能である。
+
+ # DBI
+ my $sth = $DB->prepare(q{SELECT * FROM test LIMIT ?, ?});
+ $sth->bind_param(1, $limit, { TYPE => SQL_INTEGER });
+ $sth->bind_param(2, $offset, { TYPE => SQL_INTEGER });
+ $sth->execute;
+ # TL
+ my $sth = $DB->execute(q{SELECT * FROM test LIMIT ??}, [$limit, $offset, \'SQL_INTEGER']);
+
+最後にINSERTした行のAUTO_INCREMENT値の取得などは、拡張ラッパでは制御できないので、DBIのハンドラを直接利用する。
+
+ # DBI
+ my $id = $DB->{mysql_insertid};
+ # TL
+ my $id = $DB->getDbh()->{mysql_insertid};
+
+トランザクションには C<< $DB->tx(sub{...}) >> メソッドを用いる。
+DBセットを指定する時には C<< $DB->tx(dbset_name=>sub{...}) >> となる。
+渡したコードをトランザクション内で実行する。 
+die なしにコードを抜けた時に自動的にコミットされる。 
+途中で die した場合にはトランザクションはロールバックされる。 
+
+ # DBI
+ $DB->do(q{BEGIN WORK});
+ #   do something.
+ $DB->commit;
+ 
+ # TL
+ $DB->tx(sub{
+   # do something.
+ });
+
+C<begin()> メソッドも実装はされているがその使用は非推奨である。 
+また、 C<< $DB->execute(q{BEGIN WORK}); >> は無効にされている。 
 
 =head2 拡張プレースホルダ詳細
 
@@ -1156,7 +1716,7 @@ Tripletail::DB オブジェクトを取得。
 引数にはIniで設定したグループ名を渡す。
 引数省略時は 'DB' グループが使用される。
 
-L<< $TL->startCgi|Tripletail/"startCgi" >> /  L<< $TL->errorTrap|Tripletail/"errorTrap" >> の関数内でDBオブジェクトを取得する場合に使用する。
+L<< $TL->startCgi|Tripletail/"startCgi" >> /  L<< $TL->trapError|Tripletail/"trapError" >> の関数内でDBオブジェクトを取得する場合に使用する。
 
 =item C<< $TL->newDB >>
 
@@ -1174,7 +1734,7 @@ L<< $TL->startCgi|Tripletail/"startCgi" >> /  L<< $TL->errorTrap|Tripletail/"err
 
 DBに接続する。
 
-L<< $TL->startCgi|Tripletail/"startCgi" >> /  L<< $TL->errorTrap|Tripletail/"errorTrap" >> の関数内でDBオブジェクトを取得する場合には自動的に接続が管理されるため、このメソッドを呼び出してはならない。
+L<< $TL->startCgi|Tripletail/"startCgi" >> /  L<< $TL->trapError|Tripletail/"trapError" >> の関数内でDBオブジェクトを取得する場合には自動的に接続が管理されるため、このメソッドを呼び出してはならない。
 
 L<< $TL->newDB|"$TL->newDB" >> で作成した Tripletail::DB オブジェクトに関しては、このメソッドを呼び出し、DBへ接続する必要がある。
 
@@ -1184,25 +1744,25 @@ connect時には、AutoCommit 及び RaiseError オプションは 1 が指定
 
 DBから切断する。
 
-L<< $TL->startCgi|Tripletail/"startCgi" >> /  L<< $TL->errorTrap|Tripletail/"errorTrap" >> の関数内でDBオブジェクトを取得する場合には自動的に接続が管理されるため、このメソッドを呼び出してはならない。
+L<< $TL->startCgi|Tripletail/"startCgi" >> /  L<< $TL->trapError|Tripletail/"trapError" >> の関数内でDBオブジェクトを取得する場合には自動的に接続が管理されるため、このメソッドを呼び出してはならない。
 
 L<< $TL->newDB|"$TL->newDB" >> で作成した Tripletail::DB オブジェクトに関しては、このメソッドを呼び出し、DBへの接続を切断する必要がある。
 
-=item C<< begin >>
+=item C<< tx >>
 
-  $DB->begin
-  $DB->begin('SET_W_Trans')
+  $DB->tx(sub{...})
+  $DB->tx('SET_W_Trans' => sub{...})
 
-指定されたDBセット名でトランザクションを開始する。トランザクション名
-(DBセット名) はiniで定義されていなければならない。
-名前を省略した場合は、デフォルトのDBセットが使われるが、
-setDefaultSetによってデフォルトが選ばれていない場合にはエラーとなる。
+指定されたDBセット名でトランザクションを開始し、その中でコードを
+実行する。トランザクション名(DBセット名) はiniで定義されていな
+ければならない。名前を省略した場合は、デフォルトのDBセットが使われるが、
+setDefaultSetによってデフォルトが選ばれていない場合には例外を発生させる。
 
-CGIの中でトランザクションを開始し、終了せずにMain関数を抜けた場合は、自動的に
-rollbackされる。
-
-トランザクション実行中にこのメソッドを呼んだ場合には、エラーとなる。
-1度に開始出来るトランザクションは、1つのDBグループにつき1つだけとなる。
+コードを die なしに終了した時にトランザクションは暗黙にコミットされる。
+die した場合にはロールバックされる。
+コードの中で明示的にコミット若しくはロールバックを行うこともできる。
+明示的にコミット若しくはロールバックをした後は、 tx を抜けるまで
+DB 操作は禁止される。 この間の DB 操作は例外を発生させる。 
 
 =item C<< rollback >>
 
@@ -1216,6 +1776,36 @@ rollbackされる。
 
 現在実行中のトランザクションを確定する。
 
+=item C<< inTx >>
+
+  $DB->inTx() and die "double transaction";
+  $DB->inTx('SET_W_Trans') or die "transaction required";
+
+既にトランザクション中であるかを確認する。 
+既にトランザクション中であれば真を、 
+他にトランザクションが走っていなければ偽を返す。 
+トランザクションの指定も可能。 
+異なるDBセット名のトランザクションが実行中だった場合には
+例外を発生させる。 
+
+=item C<< begin >>
+
+  $DB->begin
+  $DB->begin('SET_W_Trans')
+
+非推奨。L</tx> を使用のこと。
+
+指定されたDBセット名でトランザクションを開始する。トランザクション名
+(DBセット名) はiniで定義されていなければならない。
+名前を省略した場合は、デフォルトのDBセットが使われるが、
+setDefaultSetによってデフォルトが選ばれていない場合には例外を発生させる。
+
+CGIの中でトランザクションを開始し、終了せずにMain関数を抜けた場合は、自動的に
+rollbackされる。
+
+トランザクション実行中にこのメソッドを呼んだ場合には、例外を発生させる。
+1度に開始出来るトランザクションは、1つのDBグループにつき1つだけとなる。
+
 =item C<< setDefaultSet >>
 
   $DB->setDefaultSet('SET_W_Trans')
@@ -1245,10 +1835,10 @@ DBセットでSQLが実行される。
 
 いずれの場合でもない場合は、L</"setDefaultSet"> で指定された
 トランザクションが使用される。
-L</"setDefaultSet"> による設定がされていない場合は、エラーとなる。
+L</"setDefaultSet"> による設定がされていない場合は、例外を発生させる。
 
 このメソッドを使用して、LOCK/UNLOCK/BEGIN/COMMITといったSQL文を
-実行してはならない。実行しようとした場合はエラーになる。
+実行してはならない。実行しようとした場合は例外を発生させる。
 代わりに専用のメソッドを使用する事。
 
 =item C<< selectAllHash >>
@@ -1309,7 +1899,7 @@ SELECT結果の最初の１行を配列へのリファレンスで返す。
 のDBセットが選ばれる。CGIの中でロックした場合は、 L<Main関数|Tripletail/"Main関数">
 を抜けた時点で自動的にunlockされる。
 
-ロック実行中にこのメソッドを呼んだ場合には、エラーとなる。
+ロック実行中にこのメソッドを呼んだ場合には、例外を発生させる。
 1度に開始出来るロックは、1つのDBグループにつき1つだけとなる。
 
 =item C<< unlock >>
@@ -1317,7 +1907,7 @@ SELECT結果の最初の１行を配列へのリファレンスで返す。
   $DB->unlock
 
 UNLOCK TABLES を実行する。
-ロックがかかっていない場合はエラーとなる。
+ロックがかかっていない場合は例外を発生させる。
 
 =item C<< setBufferSize >>
 
@@ -1352,6 +1942,12 @@ DBセット内のDBハンドルを返す。
 ネイティブのDBハンドルを使用してクエリを発行した場合、デバッグ機能（プロファイリング等）の機能は使用できません。
 また、トランザクションやロック状態の管理もフレームワークで行えなくなるため、注意して使用する必要があります。
 
+=item getLastInsertId
+
+  $id = $DB->getLastInsertId()
+
+セッション内の最後の自動採番の値を取得. 
+
 =back
 
 =head3 C<Tripletail::DB::STH> メソッド
@@ -1453,7 +2049,7 @@ C<< /* foo.pl:111 [DB.R_Transaction1.DBR1] */ >> のようなコメントを挿
   type = mysql
 
 DBの種類を選択する。
-mysql, pgsql, oracleが使用可能。
+mysql, pgsql, oracle, sqlite, mssql が使用可能。
 必須項目。
 
 =item C<< defaultset >>
@@ -1498,6 +2094,36 @@ DBに接続する際のパスワードを設定する。
 
 =back
 
+=head3 SQL Server 設定
+
+試験的に SQL Server との接続が実装されています. 
+DBD::ODBC と, Linux であれば unixODBC + freetds で, Windows であれば
+組み込みの ODBC マネージャで動作します. 
+
+設定例:
+ 
+ # <tl.ini>
+ [DB]
+ type=mssql
+ defaultset=SET_W_Trans
+ SET_W_Trans=CON_RW
+ [CON_RW]
+ # dbname に ODBC-dsn を設定.
+ dbname=test
+ user=test
+ password=test
+ # freetds経由の時は, そちらのServernameも指定.
+ tdsname=tds_test
+
+freetdsでの接続文字コードの設定は F<freetds.conf> で
+設定します. 
+
+ ;; <freetds.conf>
+ [tds_test]
+ host = 10.0.0.1
+ ;;port = 1433
+ tds version = 7.0
+ client charset = UTF-8
 
 =head1 SEE ALSO
 
